@@ -21,9 +21,18 @@
 #ifndef BT_PEERCONNECTOR_H
 #define BT_PEERCONNECTOR_H
 
-#include <QSet>
+#include <QSharedPointer>
 #include <ktorrent_export.h>
 #include <util/constants.h>
+#include <util/resourcemanager.h>
+
+#if (QT_VERSION < QT_VERSION_CHECK(4, 7, 0))
+template <class T>
+uint qHash(const QSharedPointer<T> &ptr)
+{
+	return qHash<T>(ptr.data());
+}
+#endif
 
 
 namespace bt
@@ -34,9 +43,8 @@ namespace bt
 	/**
 		Class which connects to a peer.
 	*/
-	class KTORRENT_EXPORT PeerConnector : public QObject
+	class KTORRENT_EXPORT PeerConnector : public Resource
 	{
-		Q_OBJECT
 	public:
 		enum Method
 		{
@@ -55,21 +63,24 @@ namespace bt
 		/// Start connecting
 		void start();
 		
-		/// Stop connecting
-		void stop();
+		/**
+		 * Set the maximum number of active PeerConnectors allowed
+		 */
+		static void setMaxActive(Uint32 mc); 
+		
+		typedef QSharedPointer<PeerConnector> Ptr;
+		typedef QWeakPointer<PeerConnector> WPtr;
+		
+		/// Set a weak pointer to this object
+		void setWeakPointer(WPtr ptr);
+		
 		
 	private:
-		void start(Method method);
+		virtual void acquired();
 		
 	private:
-		QSet<Method> tried_methods;
-		Method current_method;
-		QString ip;
-		Uint16 port;
-		bool local;
-		PeerManager* pman;
-		Authenticate* auth;
-		bool stopping;
+		class Private;
+		Private* d;
 	};
 
 }
