@@ -44,7 +44,7 @@ namespace utp
 	}
 	
 	Connection::Connection(bt::Uint16 recv_connection_id, Type type, const net::Address& remote, Transmitter* transmitter) 
-		: transmitter(transmitter)
+		: transmitter(transmitter),timer_id(-1)
 	{
 		stats.type = type;
 		stats.remote = remote;
@@ -77,6 +77,7 @@ namespace utp
 		stats.packets_sent = 0;
 		stats.bytes_lost = 0;
 		stats.packets_lost = 0;
+		stats.readable = stats.writeable = false;
 		
 		connect(this,SIGNAL(doDelayedStartTimer()),this,SLOT(delayedStartTimer()),Qt::QueuedConnection);
 		if (type == INCOMING)
@@ -262,16 +263,35 @@ namespace utp
 				break;
 		}
 		
+		checkState();
 		startTimer();
 		return stats.state;
 	}
 	
+	void Connection::checkState()
+	{
+		// Check if we have become readable or writeable, and notify if necessary
+		bool r = local_wnd->size() > 0 || stats.state == CS_CLOSED;
+		bool w = remote_wnd->availableSpace() > 0 && stats.state == CS_CONNECTED;
+		bool r_changed = !stats.readable && r;
+		bool w_changed = !stats.writeable && w;
+		stats.readable = r;
+		stats.writeable = w;
+		
+		mutex.unlock();
+		// Temporary unlock the mutex to avoid a deadlock
+		if (r_changed || w_changed)
+			transmitter->stateChanged(self.toStrongRef(),r_changed,w_changed);
+		mutex.lock();
+	}
+
+	
 	void Connection::checkIfClosed()
 	{
 		// Check if we need to go to the closed state
 		// We can do this if all our packets have been acked and the local window
 		// has been fully read
-		if (remote_wnd->allPacketsAcked() && local_wnd->isEmpty())
+		if (stats.state == CS_FINISHED && remote_wnd->allPacketsAcked() && local_wnd->isEmpty())
 		{
 			stats.state = CS_CLOSED;
 			Out(SYS_UTP|LOG_NOTICE) << "UTP: Connection " << stats.recv_connection_id << "|" << stats.send_connection_id << " closed " << endl;
@@ -323,7 +343,7 @@ namespace utp
 		}
 		
 	
-		if (!transmitter->sendTo(ba,stats.remote,receiveConnectionID()))
+		if (!transmitter->sendTo(self.toStrongRef(),ba))
 			throw TransmissionError(__FILE__,__LINE__);
 		
 		last_packet_sent = tv;
@@ -407,6 +427,7 @@ namespace utp
 		// first put data in the output buffer then send packets
 		bt::Uint32 ret = output_buffer.write(data,len);
 		sendPackets();
+		stats.writeable = !output_buffer.full();
 		return ret;
 	}
 	
@@ -442,12 +463,9 @@ namespace utp
 		else
 			sendState();
 	}
-
-	int Connection::sendDataPacket(const QByteArray& packet)
+	
+	void Connection::sendDataPacket(const QByteArray& packet, Uint16 seq_nr, const utp::TimeValue& now)
 	{
-		bt::Uint32 to_send = packet.size();
-		TimeValue now;
-		
 		bt::Uint32 extension_length = 0;
 		bt::Uint32 sack_bits = local_wnd->selectiveAckBits();
 		if (sack_bits > 0)
@@ -462,51 +480,7 @@ namespace utp
 		hdr.timestamp_microseconds = now.timestampMicroSeconds();
 		hdr.timestamp_difference_microseconds = stats.reply_micro;
 		hdr.wnd_size = stats.last_window_size_transmitted = local_wnd->availableSpace();
-		hdr.seq_nr = stats.seq_nr;
-		hdr.ack_nr = local_wnd->lastSeqNr();
-		hdr.write((bt::Uint8*)ba.data());
-		
-		if (extension_length > 0)
-		{
-			bt::Uint8* ptr = (bt::Uint8*)(ba.data() + Header::size());
-			SelectiveAck sack;
-			sack.extension = ptr[0] = 0;
-			sack.length = ptr[1] = extension_length - 2;
-			sack.bitmask = ptr + 2;
-			local_wnd->fillSelectiveAck(&sack);
-		}
-		
-		memcpy(ba.data() + Header::size() + extension_length,packet.data(),to_send);
-		if (!transmitter->sendTo(ba,stats.remote,receiveConnectionID()))
-			throw TransmissionError(__FILE__,__LINE__);
-		
-		last_packet_sent = now;
-		stats.packets_sent++;
-		remote_wnd->addPacket(packet,stats.seq_nr,bt::Now());
-		stats.seq_nr++;
-		startTimer();
-		return to_send;
-	}
-
-	void Connection::retransmit(const QByteArray& packet, Uint16 p_seq_nr)
-	{
-		TimeValue now;
-		
-		bt::Uint32 extension_length = 0;
-		bt::Uint32 sack_bits = local_wnd->selectiveAckBits();
-		if (sack_bits > 0)
-			extension_length += 2 + qMin(sack_bits / 8,(bt::Uint32)4);
-		
-		QByteArray ba(Header::size() + extension_length + packet.size(),0);
-		Header hdr;
-		hdr.version = 1;
-		hdr.type = ST_DATA;
-		hdr.extension = extension_length == 0 ? 0 : SELECTIVE_ACK_ID;
-		hdr.connection_id = stats.send_connection_id;
-		hdr.timestamp_microseconds = now.timestampMicroSeconds();
-		hdr.timestamp_difference_microseconds = stats.reply_micro;
-		hdr.wnd_size = stats.last_window_size_transmitted = local_wnd->availableSpace();
-		hdr.seq_nr = p_seq_nr;
+		hdr.seq_nr = seq_nr;
 		hdr.ack_nr = local_wnd->lastSeqNr();
 		hdr.write((bt::Uint8*)ba.data());
 		
@@ -521,14 +495,31 @@ namespace utp
 		}
 		
 		memcpy(ba.data() + Header::size() + extension_length,packet.data(),packet.size());
-		startTimer();
-		if (!transmitter->sendTo(ba,stats.remote,receiveConnectionID()))
+		if (!transmitter->sendTo(self.toStrongRef(),ba))
 			throw TransmissionError(__FILE__,__LINE__);
 		
 		last_packet_sent = now;
 		stats.packets_sent++;
 	}
 
+
+	void Connection::sendDataPacket(const QByteArray& packet)
+	{
+		TimeValue now;
+		sendDataPacket(packet,stats.seq_nr,now);
+		
+		remote_wnd->addPacket(packet,stats.seq_nr,bt::Now());
+		stats.seq_nr++;
+		startTimer();
+	}
+
+	void Connection::retransmit(const QByteArray& packet, Uint16 p_seq_nr)
+	{
+		TimeValue now;
+		sendDataPacket(packet, p_seq_nr, now);
+		startTimer();
+	}
+
 	bt::Uint32 Connection::bytesAvailable() const
 	{
 		QMutexLocker lock(&mutex);
@@ -556,27 +547,30 @@ namespace utp
 			sendState();
 		
 		stats.bytes_received += ret;
+		stats.readable = local_wnd->size() > 0;
 		return ret;
 	}
 
 	
 	bool Connection::waitUntilConnected()
 	{
-		mutex.lock();
+		QMutexLocker lock(&mutex);
+		if (stats.state == CS_CONNECTED)
+			return true;
+		
 		connected.wait(&mutex);
-		bool ret = stats.state == CS_CONNECTED;
-		mutex.unlock();
-		return ret;
+		return stats.state == CS_CONNECTED;
 	}
 
 
-	bool Connection::waitForData()
+	bool Connection::waitForData(Uint32 timeout)
 	{
-		mutex.lock();
-		data_ready.wait(&mutex);
-		bool ret = local_wnd->size() > 0;
-		mutex.unlock();
-		return ret;
+		QMutexLocker lock(&mutex);
+		if (local_wnd->size() > 0)
+			return true;
+		
+		data_ready.wait(&mutex,timeout == 0 ? ULONG_MAX : timeout);
+		return local_wnd->size() > 0;
 	}
 
 
@@ -637,11 +631,16 @@ namespace utp
 					sendState();
 				}
 				break;
-			case CS_CLOSED:
 			case CS_IDLE:
 				startTimer();
 				break;
+			case CS_CLOSED:
+				break;
 		}
+		
+		checkState();
+		if (stats.state == CS_CLOSED)
+			transmitter->closed(self.toStrongRef());
 	}
 
 	void Connection::dumpStats()
@@ -669,19 +668,14 @@ namespace utp
 		if (QThread::currentThread() != thread())
 			emit doDelayedStartTimer();
 		else
-			timer.start(stats.timeout,this);
+			delayedStartTimer();
 	}
 	
 	void Connection::delayedStartTimer()
 	{
-		timer.start(stats.timeout,this);
+		if (timer_id != -1)
+			transmitter->cancelTimer(timer_id);
+		timer_id = transmitter->scheduleTimer(self.toStrongRef(),stats.timeout);
 	}
-
-	void Connection::timerEvent(QTimerEvent* event)
-	{
-		Q_UNUSED(event);
-		handleTimeout();
-	}
-
 }
 
