@@ -22,46 +22,16 @@
 #define UTP_UTPSERVER_H
 
 #include <QThread>
-#include <QSocketNotifier>
-#include <boost/tuple/tuple.hpp>
-#include <net/socket.h>
-#include <net/poll.h>
-#include <net/wakeuppipe.h>
-#include <util/ptrmap.h>
 #include <interfaces/serverinterface.h>
-#include <ktorrent_export.h>
-#include "connection.h"
-#include "pollpipe.h"
+#include <net/address.h>
+#include <net/poll.h>
+#include <utp/connection.h>
 
 
 namespace utp
 {
-	class UTPServerThread;
 	class UTPSocket;
-	class UTPServer;
 	
-	/**
-		Utility class used by UTPServer to make sure that ServerInterface::newConnection is called
-		from the main thread and not from UTP thread (which is dangerous).
-	*/
-	class KTORRENT_EXPORT MainThreadCall : public QObject
-	{
-		Q_OBJECT
-	public:
-		MainThreadCall(UTPServer* server);
-		virtual ~MainThreadCall();
-		
-	public slots:
-		/**
-			Calls UTPServer::handlePendingConnections, this should be executed in 
-			the main thread.
-		*/
-		void handlePendingConnections();
-		
-	private:
-		UTPServer* server;
-	};
-
 	/**
 		Implements the UTP server. It listens for UTP packets and manages all connections.
 	*/
@@ -73,21 +43,18 @@ namespace utp
 		virtual ~UTPServer();
 		
 		/// Enabled creating sockets (tests need to have this disabled)
-		void setCreateSockets(bool on) {create_sockets = on;}
+		void setCreateSockets(bool on);
 		
 		virtual bool changePort(bt::Uint16 port);
 		
 		/// Send a packet to some host
-		virtual bool sendTo(const QByteArray & data,const net::Address & addr,quint16 conn_id);
+		virtual bool sendTo(Connection::Ptr conn,const QByteArray & data);
 		
 		/// Setup a connection to a remote address
-		Connection* connectTo(const net::Address & addr);
+		Connection::WPtr connectTo(const net::Address & addr);
 		
-		/// Attach a socket to a Connection
-		void attach(UTPSocket* socket,Connection* conn);
-		
-		/// Detach a socket to a Connection
-		void detach(UTPSocket* socket,Connection* conn);
+		/// Get the last accepted connection (Note: for unittest purposes)
+		Connection::WPtr acceptedConnection();
 		
 		/// Start the UTP server
 		void start();
@@ -96,7 +63,7 @@ namespace utp
 		void stop();
 		
 		/// Prepare the server for polling
-		void preparePolling(net::Poll* p,net::Poll::Mode mode,Connection* conn);
+		void preparePolling(net::Poll* p,net::Poll::Mode mode,Connection::Ptr conn);
 		
 		/// Set the TOS byte
 		void setTOS(bt::Uint8 type_of_service);
@@ -111,62 +78,25 @@ namespace utp
 		void handlePendingConnections();
 		
 	protected:
-		bool bind(const net::Address & addr);
 		virtual void handlePacket(const QByteArray & packet,const net::Address & addr);
-		void syn(const PacketParser & parser,const QByteArray & data,const net::Address & addr);
-		void reset(const Header* hdr);
-		void clearDeadConnections();
-		void wakeUpPollPipes();
-		Connection* find(quint16 conn_id);
-		virtual void timerEvent(QTimerEvent* event);
-		virtual void customEvent(QEvent* event);
+		virtual void stateChanged(Connection::Ptr conn, bool readable, bool writeable);
+		virtual void closed(Connection::Ptr conn);
+		virtual void customEvent(QEvent* ev);
+		virtual void timerEvent(QTimerEvent* ev);
+		virtual int scheduleTimer(Connection::Ptr conn, Uint32 timeout);
+		virtual void cancelTimer(int timer_id);
 		
 	signals:
 		void handlePendingConnectionsDelayed();
-		void accepted(Connection* conn);
+		/// Emitted when a connection is accepted if creating sockets is disabled
+		void accepted();
 		
 	private slots:
-		void onAccepted(Connection* conn);
-		void readPacket(int fd);
-		void writePacket(int fd);
-		
-	public slots:
 		void cleanup();
-		
+
 	private:
-		typedef bt::PtrMap<quint16,Connection>::iterator ConItr;
-		
-		struct PollPipePair
-		{
-			PollPipe::Ptr read_pipe;
-			PollPipe::Ptr write_pipe;
-			
-			PollPipePair();
-			
-			void testRead(ConItr b,ConItr e);
-			void testWrite(ConItr b,ConItr e);
-		};
-		
-		typedef bt::PtrMap<net::Poll*,PollPipePair>::iterator PollPipePairItr;
-		typedef boost::tuple<QByteArray,net::Address,quint16> OutputQueueEntry;
-		
-	private:
-		net::Socket* sock;
-		bool running;
-		bt::PtrMap<quint16,Connection> connections;
-		QList<Connection*> dead_connections;
-		bt::PtrMap<Connection*,UTPSocket> alive_connections;
-		UTPServerThread* utp_thread;
-		QMutex mutex;
-		bt::PtrMap<net::Poll*,PollPipePair> poll_pipes;
-		bool create_sockets;
-		bt::Uint8 tos;
-		QList<OutputQueueEntry> output_queue;
-		QSocketNotifier* read_notifier;
-		QSocketNotifier* write_notifier;
-		QBasicTimer timer;
-		QList<mse::StreamSocket*> pending;
-		MainThreadCall* mtc;
+		class Private;
+		Private* d;
 	};
 
 }
